import { IObservableValue } from 'mobx';
import { AxisLinePosition, AxisTick, AxisTicksDimensions } from '../utils/axis_utils';
import { XDomain } from '../domains/x_domain';
import { YDomain } from '../domains/y_domain';
import { LegendItem } from '../legend/legend';
import { AreaGeometry, BarGeometry, GeometryValue, IndexedGeometry, LineGeometry, PointGeometry } from '../rendering/rendering';
import { DataSeriesColorsValues, FormattedDataSeries, RawDataSeries } from '../utils/series';
import { AnnotationSpec, AreaSeriesSpec, AxisSpec, BarSeriesSpec, BasicSeriesSpec, DomainRange, LineSeriesSpec, Position, Rendering, Rotation } from '../utils/specs';
import { Theme } from '../../../utils/themes/theme';
import { Dimensions } from '../../../utils/dimensions';
import { Domain } from '../../../utils/domain';
import { AnnotationId, AxisId, GroupId, SpecId } from '../../../utils/ids';
import { TooltipType, TooltipValue, TooltipValueFormatter } from '../utils/interactions';
import { Scale } from '../../../utils/scales/scales';
import { CursorEvent } from '../../../specs/settings';
import { AnnotationDimensions } from '../annotations/annotation_utils';
import { TooltipPosition } from '../crosshair/crosshair_utils';
import { BrushExtent, Transform } from './utils';
export interface Point {
    x: number;
    y: number;
}
export interface SeriesDomainsAndData {
    xDomain: XDomain;
    yDomain: YDomain[];
    splittedDataSeries: RawDataSeries[][];
    formattedDataSeries: {
        stacked: FormattedDataSeries[];
        nonStacked: FormattedDataSeries[];
    };
    seriesColors: Map<string, DataSeriesColorsValues>;
}
export declare type ElementClickListener = (values: GeometryValue[]) => void;
export declare type ElementOverListener = (values: GeometryValue[]) => void;
export declare type BrushEndListener = (min: number, max: number) => void;
export declare type LegendItemListener = (dataSeriesIdentifiers: DataSeriesColorsValues | null) => void;
export declare type CursorUpdateListener = (event?: CursorEvent) => void;
/**
 * Listener to be called when chart render state changes
 *
 * `isRendered` value is `true` when rendering is complete and `false` otherwise
 */
export declare type RenderChangeListener = (isRendered: boolean) => void;
export declare type BasicListener = () => undefined | void;
export declare const isDuplicateAxis: ({ position, title }: AxisSpec, { tickLabels }: AxisTicksDimensions, tickMap: Map<AxisId, AxisTicksDimensions>, specMap: Map<AxisId, AxisSpec>) => boolean;
export declare class ChartStore {
    constructor(id?: string);
    debug: boolean;
    id: string;
    specsInitialized: IObservableValue<boolean>;
    chartInitialized: IObservableValue<boolean>;
    legendInitialized: IObservableValue<boolean>;
    enableHistogramMode: IObservableValue<boolean>;
    parentDimensions: Dimensions;
    chartDimensions: Dimensions;
    chartTransform: Transform;
    isBrushing: IObservableValue<boolean>;
    brushExtent: BrushExtent;
    resizeDebounce: number;
    chartRotation: Rotation;
    chartRendering: Rendering;
    chartTheme: Theme;
    hideDuplicateAxes: boolean;
    axesSpecs: Map<AxisId, AxisSpec>;
    axesTicksDimensions: Map<AxisId, AxisTicksDimensions>;
    axesPositions: Map<AxisId, Dimensions>;
    axesVisibleTicks: Map<AxisId, AxisTick[]>;
    axesTicks: Map<AxisId, AxisTick[]>;
    axesGridLinesPositions: Map<AxisId, AxisLinePosition[]>;
    annotationSpecs: Map<AnnotationId, AnnotationSpec>;
    annotationDimensions: import("mobx").ObservableMap<AnnotationId, AnnotationDimensions>;
    seriesSpecs: Map<SpecId, BasicSeriesSpec>;
    isChartEmpty: IObservableValue<boolean>;
    activeChartId?: string;
    seriesDomainsAndData?: SeriesDomainsAndData;
    xScale?: Scale;
    yScales?: Map<GroupId, Scale>;
    customXDomain?: Domain | DomainRange;
    legendItems: Map<string, LegendItem>;
    highlightedLegendItemKey: IObservableValue<string | null>;
    selectedLegendItemKey: IObservableValue<string | null>;
    deselectedDataSeries: DataSeriesColorsValues[] | null;
    customSeriesColors: Map<string, string>;
    seriesColorMap: Map<string, string>;
    totalBarsInCluster?: number;
    tooltipData: import("mobx").IObservableArray<TooltipValue>;
    tooltipType: IObservableValue<TooltipType>;
    tooltipSnap: IObservableValue<boolean>;
    tooltipPosition: TooltipPosition & import("mobx").IObservableObject;
    tooltipHeaderFormatter?: TooltipValueFormatter;
    /** cursorPosition is used by tooltip, so this is a way to expose the position for other uses */
    rawCursorPosition: {
        x: number;
        y: number;
    } & import("mobx").IObservableObject;
    /** position of the cursor relative to the chart */
    cursorPosition: {
        x: number;
        y: number;
    } & import("mobx").IObservableObject;
    cursorBandPosition: Dimensions & {
        visible: boolean;
    } & import("mobx").IObservableObject;
    cursorLinePosition: Dimensions & import("mobx").IObservableObject;
    externalCursorShown: IObservableValue<boolean>;
    onElementClickListener?: ElementClickListener;
    onElementOverListener?: ElementOverListener;
    onElementOutListener?: BasicListener;
    onBrushEndListener?: BrushEndListener;
    onLegendItemOverListener?: LegendItemListener;
    onLegendItemOutListener?: BasicListener;
    onLegendItemClickListener?: LegendItemListener;
    onLegendItemPlusClickListener?: LegendItemListener;
    onLegendItemMinusClickListener?: LegendItemListener;
    onLegendItemVisibilityToggleClickListener?: LegendItemListener;
    onCursorUpdateListener?: CursorUpdateListener;
    onRenderChangeListener?: RenderChangeListener;
    geometries: {
        points: PointGeometry[];
        bars: BarGeometry[];
        areas: AreaGeometry[];
        lines: LineGeometry[];
    } | null;
    geometriesIndex: Map<any, IndexedGeometry[]>;
    geometriesIndexKeys: any[];
    highlightedGeometries: import("mobx").IObservableArray<IndexedGeometry>;
    animateData: boolean;
    /**
     * Define if the chart can be animated or not depending
     * on the global configuration and on the number of elements per series
     */
    canDataBeAnimated: boolean;
    showLegend: IObservableValue<boolean>;
    legendPosition: IObservableValue<Position>;
    showLegendDisplayValue: IObservableValue<boolean>;
    isCursorOnChart: IObservableValue<boolean>;
    chartCursor: import("mobx").IComputedValue<"default" | "pointer" | "crosshair">;
    /**
     * determine if chart is currently active
     */
    isActiveChart: import("mobx").IComputedValue<boolean>;
    /**
     * set activeChartId
     */
    setActiveChartId: (chartId?: string | undefined) => void;
    /**
     * set the x value of the cursor
     */
    setCursorValue: ((value: string | number) => void) & import("mobx").IAction;
    /**
     * x and y values are relative to the container.
     */
    setCursorPosition: ((x: number, y: number, updateCursor?: boolean) => void) & import("mobx").IAction;
    legendItemTooltipValues: import("mobx").IComputedValue<Map<string, import("../tooltip/tooltip").TooltipLegendValue>>;
    annotationTooltipState: import("mobx").IComputedValue<import("../annotations/annotation_utils").AnnotationTooltipVisibleState | import("../annotations/annotation_utils").AnnotationTooltipHiddenState | null>;
    isTooltipVisible: import("mobx").IComputedValue<boolean>;
    isCrosshairVisible: import("mobx").IComputedValue<boolean>;
    isTooltipSnapEnabled: import("mobx").IComputedValue<boolean>;
    clearTooltipAndHighlighted: ((clearCursorBand?: unknown) => void) & import("mobx").IAction;
    setShowLegend: ((showLegend: boolean) => void) & import("mobx").IAction;
    highlightedLegendItem: import("mobx").IComputedValue<LegendItem | null | undefined>;
    selectedLegendItem: import("mobx").IComputedValue<LegendItem | null | undefined>;
    onLegendItemOver: ((legendItemKey: string | null) => void) & import("mobx").IAction;
    onLegendItemOut: (() => void) & import("mobx").IAction;
    onLegendItemClick: ((legendItemKey: string) => void) & import("mobx").IAction;
    onLegendItemPlusClick: (() => void) & import("mobx").IAction;
    onLegendItemMinusClick: (() => void) & import("mobx").IAction;
    toggleSingleSeries: ((legendItemKey: string) => void) & import("mobx").IAction;
    updateHighlightedLegendItemKey: ((legendItemKey: string, deselected: boolean) => void) & import("mobx").IAction;
    toggleSeriesVisibility: ((legendItemKey: string) => void) & import("mobx").IAction;
    setSeriesColor: ((legendItemKey: string, color: string) => void) & import("mobx").IAction;
    onBrushStart: (() => void) & import("mobx").IAction;
    onBrushEnd: ((start: Point, end: Point) => void) & import("mobx").IAction;
    handleChartClick(): void;
    resetDeselectedDataSeries(): void;
    setOnElementClickListener(listener: ElementClickListener): void;
    setOnElementOverListener(listener: ElementOverListener): void;
    setOnElementOutListener(listener: BasicListener): void;
    setOnBrushEndListener(listener: BrushEndListener): void;
    setOnLegendItemOverListener(listener: LegendItemListener): void;
    setOnLegendItemOutListener(listener: BasicListener): void;
    setOnLegendItemClickListener(listener: LegendItemListener): void;
    setOnLegendItemPlusClickListener(listener: LegendItemListener): void;
    setOnLegendItemMinusClickListener(listener: LegendItemListener): void;
    setOnCursorUpdateListener(listener: CursorUpdateListener): void;
    setOnRenderChangeListener(listener: RenderChangeListener): void;
    removeElementClickListener(): void;
    removeElementOverListener(): void;
    removeElementOutListener(): void;
    removeOnLegendItemOverListener(): void;
    removeOnLegendItemOutListener(): void;
    removeOnLegendItemPlusClickListener(): void;
    removeOnLegendItemMinusClickListener(): void;
    removeOnCursorUpdateListener(): void;
    removeOnRenderChangeListener(): void;
    isBrushEnabled(): boolean;
    updateParentDimensions(width: number, height: number, top: number, left: number): void;
    addSeriesSpec(seriesSpec: BasicSeriesSpec | LineSeriesSpec | AreaSeriesSpec | BarSeriesSpec): void;
    removeSeriesSpec(specId: SpecId): void;
    /**
     * Add an axis spec to the store
     * @param axisSpec an axis spec
     */
    addAxisSpec(axisSpec: AxisSpec): void;
    removeAxisSpec(axisId: AxisId): void;
    addAnnotationSpec(annotationSpec: AnnotationSpec): void;
    removeAnnotationSpec(annotationId: AnnotationId): void;
    computeChart(): void;
}
