import { RecursivePartial } from '../commons';
import { GeometryStyle } from '../../chart_types/xy_chart/rendering/rendering';
import { Margins } from '../dimensions';
interface Visible {
    visible: boolean;
}
export interface TextStyle {
    fontSize: number;
    fontFamily: string;
    fontStyle?: string;
    fill: string;
    padding: number;
}
export interface GeometryStyle {
    stroke: string;
    strokeWidth: number;
    fill?: string;
    opacity?: number;
}
export interface SharedGeometryStyle {
    default: GeometryStyle;
    highlighted: GeometryStyle;
    unhighlighted: GeometryStyle;
}
export interface StrokeStyle {
    /** The stroke color in hex, rgba, hsl */
    stroke: string;
    /** The stroke width in pixel */
    strokeWidth: number;
}
export declare type TickStyle = StrokeStyle & Visible;
export interface StrokeDashArray {
    /** The dash array for dashed strokes */
    dash: number[];
}
export interface FillStyle {
    /** The fill color in hex, rgba, hsl */
    fill: string;
}
export interface Opacity {
    /** The opacity value from 0 to 1 */
    opacity: number;
}
export interface AxisConfig {
    axisTitleStyle: TextStyle;
    axisLineStyle: StrokeStyle;
    tickLabelStyle: TextStyle;
    tickLineStyle: TickStyle;
    gridLineStyle: {
        horizontal: GridLineConfig;
        vertical: GridLineConfig;
    };
}
export interface GridLineConfig {
    visible?: boolean;
    stroke?: string;
    strokeWidth?: number;
    opacity?: number;
    dash?: number[];
}
export interface ScalesConfig {
    /**
     * The proportion of the range that is reserved for blank space between bands.
     * A value of 0 means no blank space between bands, and a value of 1 means a bandwidth of zero.
     * A number between 0 and 1.
     */
    barsPadding: number;
    /**
     * The proportion of the range that is reserved for blank space between bands in histogramMode.
     * A value of 0 means no blank space between bands, and a value of 1 means a bandwidth of zero.
     * A number between 0 and 1.
     */
    histogramPadding: number;
}
export interface ColorConfig {
    vizColors: string[];
    defaultVizColor: string;
}
export interface LegendStyle {
    /**
     * Max width used for left/right legend
     *
     * or
     *
     * Width of `LegendItem` for top/bottom legend
     */
    verticalWidth: number;
    /**
     * Max height used for top/bottom legend
     */
    horizontalHeight: number;
    /**
     * Added buffer between label and value.
     *
     * Smaller values render a more compact legend
     */
    spacingBuffer: number;
}
export interface Theme {
    /**
     * Space btw parent DOM element and first available element of the chart (axis if exists, else the chart itself)
     */
    chartMargins: Margins;
    /**
     * Space btw the chart geometries and axis; if no axis, pads space btw chart & container
     */
    chartPaddings: Margins;
    /**
     * Global line styles.
     *
     * __Note:__ This is not used to set the color of a specific series. As such, any changes to the styles will not be reflected in the tooltip, legend, etc..
     *
     * You may use `CustomSeriesColorsMap` to assign colors to a given series or replace the `theme.colors.vizColors` colors to your desired colors.
     */
    lineSeriesStyle: LineSeriesStyle;
    /**
     * Global area styles.
     *
     * __Note:__ This is not used to set the color of a specific series. As such, any changes to the styles will not be reflected in the tooltip, legend, etc..
     *
     * You may use `CustomSeriesColorsMap` to assign colors to a given series or replace the `theme.colors.vizColors` colors to your desired colors.
     */
    areaSeriesStyle: AreaSeriesStyle;
    /**
     * Global bar styles.
     *
     * __Note:__ This is not used to set the color of a specific series. As such, any changes to the styles will not be reflected in the tooltip, legend, etc..
     *
     * You may use `CustomSeriesColorsMap` to assign colors to a given series or replace the `theme.colors.vizColors` colors to your desired colors.
     */
    barSeriesStyle: BarSeriesStyle;
    sharedStyle: SharedGeometryStyle;
    axes: AxisConfig;
    scales: ScalesConfig;
    colors: ColorConfig;
    legend: LegendStyle;
    crosshair: CrosshairStyle;
}
export declare type PartialTheme = RecursivePartial<Theme>;
export declare type DisplayValueStyle = TextStyle & {
    offsetX: number;
    offsetY: number;
};
export interface PointStyle {
    /** is the point visible or hidden */
    visible: boolean;
    /** a static stroke color if defined, if not it will use the color of the series */
    stroke?: string;
    /** the stroke width of the point */
    strokeWidth: number;
    /**  a static fill color if defined, if not it will use the color of the series */
    fill?: string;
    /** the opacity of each point on the theme/series */
    opacity: number;
    /** the radius of each point of the theme/series */
    radius: number;
}
export interface LineStyle {
    /** is the line visible or hidden ? */
    visible: boolean;
    /** a static stroke color if defined, if not it will use the color of the series */
    stroke?: string;
    /** the stroke width of the line */
    strokeWidth: number;
    /** the opacity of each line on the theme/series */
    opacity: number;
}
export interface AreaStyle {
    /** is the area is visible or hidden ? */
    visible: boolean;
    /** a static fill color if defined, if not it will use the color of the series */
    fill?: string;
    /** the opacity of each area on the theme/series */
    opacity: number;
}
export interface RectStyle {
    /** a static fill color if defined, if not it will use the color of the series */
    fill?: string;
    /** the opacity of each rect on the theme/series */
    opacity: number;
}
export interface RectBorderStyle {
    /**
     * Border visibility
     */
    visible: boolean;
    /**
     * Border stroke color
     */
    stroke?: string;
    /**
     * Border stroke width
     */
    strokeWidth: number;
    /**
     * Border stroke opacity
     */
    strokeOpacity?: number;
}
export interface BarSeriesStyle {
    rect: RectStyle;
    rectBorder: RectBorderStyle;
    displayValue: DisplayValueStyle;
}
export interface LineSeriesStyle {
    line: LineStyle;
    point: PointStyle;
}
export interface AreaSeriesStyle {
    area: AreaStyle;
    line: LineStyle;
    point: PointStyle;
}
export interface CrosshairStyle {
    band: FillStyle & Visible;
    line: StrokeStyle & Visible & Partial<StrokeDashArray>;
}
export interface LineAnnotationStyle {
    line: StrokeStyle & Opacity;
    details: TextStyle;
}
export declare type RectAnnotationStyle = StrokeStyle & FillStyle & Opacity;
export declare const DEFAULT_ANNOTATION_LINE_STYLE: LineAnnotationStyle;
export declare const DEFAULT_ANNOTATION_RECT_STYLE: RectAnnotationStyle;
export declare function mergeGridLineConfigs(axisSpecConfig: GridLineConfig, themeConfig: GridLineConfig): GridLineConfig;
export declare function mergeWithDefaultAnnotationLine(config?: Partial<LineAnnotationStyle>): LineAnnotationStyle;
export declare function mergeWithDefaultAnnotationRect(config?: Partial<RectAnnotationStyle>): RectAnnotationStyle;
/**
 * Merge theme or themes with a base theme
 *
 * priority is based on spatial order
 *
 * @param theme - primary partial theme
 * @param defaultTheme - base theme
 * @param axillaryThemes - additional themes to be merged
 */
export declare function mergeWithDefaultTheme(theme: PartialTheme, defaultTheme?: Theme, axillaryThemes?: PartialTheme[]): Theme;
export {};
