import { isEqual, cloneDeep } from 'lodash';
import { migrateFilter } from '../_migrate_filter';

describe('migrateFilter', function () {

  const oldMatchPhraseFilter = {
    match: {
      fieldFoo: {
        query: 'foobar',
        type: 'phrase'
      }
    }
  };

  const newMatchPhraseFilter = {
    match_phrase: {
      fieldFoo: {
        query: 'foobar'
      }
    }
  };

  const shouldFilter = {
    query: { bool: { should: [
      {
        bool: { must: [
          { term: { _uid: 'doc#2' } },
          { term: { _index: 'index-1' } }
        ] }
      },
      {
        bool: { must: [
          { term: { _uid: 'doc#1' } },
          { term: { _index: 'index-2' } }
        ] }
      }
    ] } }
  };

  it('should migrate match filters of type phrase', function () {
    const migratedFilter = migrateFilter(oldMatchPhraseFilter);
    expect(isEqual(migratedFilter, newMatchPhraseFilter)).toEqual(true);
  });

  it('should not modify the original filter', function () {
    const oldMatchPhraseFilterCopy = cloneDeep(oldMatchPhraseFilter);
    migrateFilter(oldMatchPhraseFilter);
    expect(isEqual(oldMatchPhraseFilter, oldMatchPhraseFilterCopy)).toEqual(true);
  });

  it('should return the original filter if no migration is necessary - empty filter', function () {
    const originalFilter = {
      match_all: {}
    };
    const migratedFilter = migrateFilter(originalFilter);
    expect(migratedFilter).toBe(originalFilter);
    expect(isEqual(migratedFilter, originalFilter)).toEqual(true);
  });

  it('should return the original filter if no migration is necessary - bool should filter', function () {
    const migratedFilter = migrateFilter(shouldFilter);
    expect(migratedFilter).toBe(shouldFilter);
    expect(isEqual(migratedFilter, shouldFilter)).toEqual(true);
  });

  it('should not fail and should return undefined', function () {
    const undefinedMigratedFilter = migrateFilter(undefined);
    expect(undefinedMigratedFilter).toEqual(undefined);
    expect(migrateFilter).not.toThrow();
  });

  it('should handle `should`, `filter`, `must` and `must_not` as an array of filters', function () {
    const query = {
      bool: {
        must: [{ exists: { field: 'founded_year' } }],
        must_not: [{ exists: { field: 'founded_year' } }],
        should: [{ exists: { field: 'founded_year' } }],
        filter: [{ exists: { field: 'founded_year' } }]
      }
    };
    const migratedQuery = migrateFilter(query);
    expect(isEqual(query, migratedQuery)).toEqual(true);
  });

  it('should handle `should`, `filter`, `must` and `must_not` as a single filter', function () {
    const query = {
      bool: {
        must: { exists: { field: 'founded_year' } },
        must_not: { exists: { field: 'founded_year' } },
        should: { exists: { field: 'founded_year' } },
        filter: { exists: { field: 'founded_year' } }
      }
    };
    const migratedQuery = migrateFilter(query);
    expect(isEqual(query, migratedQuery)).toEqual(true);
  });
});
