import { QueryBuilderFactory } from '../query_builder';
import { getQueryFixture } from '../fixtures/query_fixture';
import { differentKindOfFilters, differentKindOfNegatedFilters } from '../fixtures/filter_list';
import { map, omit } from 'lodash';

// time in epoch millis;
const defaultStartTime = 1157112000000;
const defaultEndTime = 1283688000000;

const defaultQuery = { match_all: {} };
const queryBuilder = new QueryBuilderFactory();

describe('QueryBuilder', () => {

  const assertQuery = (queryType, expected) => {
    const { filters, queries, time } = getQueryFixture(queryType);
    const query = queryBuilder(filters, queries, time);
    expect(query).toEqual(expected);
  };

  it('empty', () => {
    const expected = {
      query: {
        bool: {
          must: [
            defaultQuery,
            {
              range: {
                date: {
                  gte: defaultStartTime,
                  lte: defaultEndTime,
                  format: 'epoch_millis'
                }
              }
            }
          ],
          must_not: []
        }
      }
    };
    return assertQuery('empty', expected);
  });

  it('saved search', () => {
    const expected = {
      query: {
        bool: {
          must: [
            defaultQuery,
            {
              query_string: {
                query: 'funded_year:>2010',
                analyze_wildcard: true
              }
            },
            {
              range: {
                date: {
                  gte: defaultStartTime,
                  lte: defaultEndTime,
                  format: 'epoch_millis'
                }
              }
            }
          ],
          must_not: []
        }
      }
    };

    return assertQuery('saved_search', expected);
  });

  it('query filter regular', () => {
    const expected = {
      query: {
        bool: {
          must: [
            {
              query_string: {
                query: 'dog',
                analyze_wildcard: true
              }
            },
            defaultQuery,
            {
              range: {
                date: {
                  gte: defaultStartTime,
                  lte: defaultEndTime,
                  format: 'epoch_millis'
                }
              }
            }
          ],
          must_not: []
        }
      }
    };

    return assertQuery('regular_query', expected);
  });


  it('query filter negated', () => {
    const expected = {
      query: {
        bool: {
          must: [
            defaultQuery,
            {
              range: {
                date: {
                  gte: defaultStartTime,
                  lte: defaultEndTime,
                  format: 'epoch_millis'
                }
              }
            }
          ],
          must_not: [
            {
              query_string: {
                query: 'dog',
                analyze_wildcard: true // added by the advanced setting "query:queryString:options"
              }
            }
          ]
        }
      }
    };

    return assertQuery('negated_query', expected);

  });

  it('different types of filters', () => {
    const expected = {
      query: {
        bool: {
          must: [
            ...map(differentKindOfFilters, f => omit(f, 'meta')),
            defaultQuery,
            {
              range: {
                date: {
                  gte: defaultStartTime,
                  lte: defaultEndTime,
                  format: 'epoch_millis'
                }
              }
            }
          ],
          must_not: []
        }
      }
    };

    return assertQuery('different_filter_query', expected);
  });

  it('different types of filters negated', () => {
    const expected = {
      query: {
        bool: {
          must: [
            defaultQuery,
            {
              range: {
                date: {
                  gte: defaultStartTime,
                  lte: defaultEndTime,
                  format: 'epoch_millis'
                }
              }
            }
          ],
          must_not: map(differentKindOfNegatedFilters.filter(filter => !filter.meta.disabled), (f) => omit(f, 'meta'))
        }
      }
    };

    return assertQuery('different_negated_filter_query', expected);
  });
});

