const { isUndefined, omit, isFunction, each } = require('lodash');
const { DecorateQueryProvider } = require('./_decorate_query');
const { migrateFilter } = require('./_migrate_filter');

/**
* Create a filter that can be reversed for filters with negate set
* @param {boolean} reverse This will reverse the filter. If true then
*                          anything where negate is set will come
*                          through otherwise it will filter out
* @returns {function}
*/
const filterNegate = function (reverse) {
  return function (filter) {
    if (isUndefined(filter.meta) || isUndefined(filter.meta.negate)) return !reverse;
    return filter.meta && filter.meta.negate === reverse;
  };
};

const removeDisabled = filter => !filter.meta || !filter.meta.disabled;

/**
* Translate a filter into a query to support es 3+
* @param  {Object} filter - The filter to translate
* @return {Object} the query version of that filter
*/
const translateToQuery = function (filter) {
  if (!filter) return;

  if (filter.query) {
    return filter.query;
  }

  if (filter.range) {
    return {
      range: filter.range
    };
  }

  return filter;
};

/**
* Clean out any invalid attributes from the filters
* @param {object} filter The filter to clean
* @returns {object}
*/
const cleanFilter = function (filter) {
  return omit(filter, ['meta', '$state']);
};

function QueryBuilderFactory(Private) {

  let decorateQuery = null;

  if (isFunction(Private)) {
    decorateQuery = Private(DecorateQueryProvider);
  }
  else {
    decorateQuery = DecorateQueryProvider;
  }

  /*
  * The parameter savedSearch should be a reference to a SavedSearch
    * instance, not a SavedSearch id
    */
  return function queryBuilder(filters = [], queries, time) {
    const request = {};

    // logic copied from src/ui/public/courier/data_source/_abstract.js
    each(filters, function (filter) {
      if (filter.query) {
        decorateQuery(filter.query);
      } else if (filter.query_string) {
        decorateQuery(filter);
      }
    });

    request.query = {
      bool: {
        must: (
          filters
            .filter(removeDisabled)
            .filter(filterNegate(false))
            .map(translateToQuery)
            .map(cleanFilter)
            .map(migrateFilter)
        ),
        must_not: (
          filters
            .filter(removeDisabled)
            .filter(filterNegate(true))
            .map(translateToQuery)
            .map(cleanFilter)
            .map(migrateFilter)
        )
      }
    };

    each(queries, (q) => {
      decorateQuery(q);
      request.query.bool.must.push(q);
    });

    // add time
    if (time) {
      request.query.bool.must.push(time);
    }
    return request;
  };
}

module.exports = {
  'QueryBuilderFactory': QueryBuilderFactory
};