import React from 'react';
import { EuiFlexGroup, EuiFlexItem, EuiIcon, EuiLoadingChart, EuiTextColor } from '@elastic/eui';
import { AnnotationDomainTypes, AreaSeries, Axis, BarSeries, Chart, getAnnotationId, getAxisId, getSpecId, LineAnnotation, LineSeries, Position, ScaleType, Settings } from '@elastic/charts';
import { dateTickFormatter } from '../TimeUtils';
import moment from 'moment';
import { COLOURS } from '../constants';
import '@elastic/eui/dist/eui_theme_light.css';
import '@elastic/charts/dist/theme_only_light.css';
import './Graph.less';
export default class Graph extends React.PureComponent {
    constructor(props) {
        super(props);
        this.getAnnotations = () => {
            const series = [];
            const styleCritical = {
                line: {
                    strokeWidth: 3,
                    stroke: COLOURS.red,
                    opacity: 1
                }
            };
            const styleWarning = {
                line: {
                    strokeWidth: 3,
                    stroke: COLOURS.orange,
                    opacity: 1
                }
            };
            series.push(<LineAnnotation key={0} annotationId={getAnnotationId(`${this.props.seriesLabels[0]}-critical`)} domainType={AnnotationDomainTypes.XDomain} dataValues={this.props.annotations && this.props.annotations
                .filter(annotation => annotation.severity === 'critical')
                .map(annotation => ({
                dataValue: annotation.dataValue,
                details: annotation.details,
                header: annotation.header
            }))} style={styleCritical} marker={<EuiIcon type="alert"/>}/>);
            series.push(<LineAnnotation key={1} annotationId={getAnnotationId(`${this.props.seriesLabels[0]}-warning`)} domainType={AnnotationDomainTypes.XDomain} dataValues={this.props.annotations && this.props.annotations
                .filter(annotation => annotation.severity === 'warning')
                .map(annotation => ({
                dataValue: annotation.dataValue,
                details: annotation.details,
                header: annotation.header
            }))} style={styleWarning} marker={<EuiIcon type="iInCircle"/>}/>);
            return series;
        };
        this.getAreaSeries = () => {
            const specId = getSpecId(this.props.areaLabel || '');
            return (<AreaSeries key={0} id={specId} xScaleType={ScaleType.Time} yScaleType={ScaleType.Linear} xAccessor={0} yAccessors={[2]} y0Accessors={[1]} data={this.props.area} yScaleToDataExtent={true} customSeriesColors={new Map([[{ colorValues: [], specId }, this.props.areaColour ? this.props.areaColour : COLOURS.lightblue]])} areaSeriesStyle={{ line: { visible: false }, point: { visible: false } }}/>);
        };
        this.getLineSeries = (numberOfSeries) => {
            const series = [];
            for (let i = 0; i < numberOfSeries; i++) {
                const specId = getSpecId(this.props.seriesLabels[i]);
                series.push(<LineSeries key={i} id={specId} xScaleType={this.props.xLabelType || ScaleType.Linear} yScaleType={ScaleType.Linear} xAccessor={0} yAccessors={[i + 1]} data={this.props.data} customSeriesColors={new Map([[{ colorValues: [], specId },
                        this.props.lineColours && this.props.lineColours[i] ? this.props.lineColours[i] : COLOURS.blue]])} yScaleToDataExtent={true} lineSeriesStyle={{ point: { visible: false } }}/>);
            }
            return series;
        };
        this.getBarSeries = (numberOfSeries) => {
            const series = [];
            for (let i = 0; i < numberOfSeries; i++) {
                series.push(<BarSeries key={i} id={getSpecId(this.props.seriesLabels[i])} xScaleType={ScaleType.Linear} yScaleType={ScaleType.Linear} xAccessor={0} yAccessors={[i + 1]} data={this.props.data}/>);
            }
            return series;
        };
        this.getXTickFormatter = () => {
            if (this.props.xTickFormatter) {
                return this.props.xTickFormatter;
            }
            else if (this.props.type === 'bar' || !this.props.data) {
                return String;
            }
            else {
                const start = moment(this.props.data[0][0] || 0);
                const end = moment(this.props.data[this.props.data.length - 1][0] || 0);
                return x => dateTickFormatter(x, start, end);
            }
        };
        this.state = {};
    }
    render() {
        if (this.props.errorMessage) {
            return (<EuiFlexGroup justifyContent="spaceAround" alignItems="center" style={{ width: '100%', ...this.props.style }}>
          <EuiFlexItem grow={false}>
            <EuiTextColor color="danger">
              {this.props.errorMessage}
            </EuiTextColor>
          </EuiFlexItem>
        </EuiFlexGroup>);
        }
        else if (this.props.isLoading) {
            return (<EuiFlexGroup justifyContent="spaceAround" alignItems="center" style={{ width: '100%', ...this.props.style }}>
          <EuiFlexItem grow={false}>
            <EuiLoadingChart size="xl"/>
          </EuiFlexItem>
        </EuiFlexGroup>);
        }
        else if (this.props.data && this.props.data.length) {
            return (<React.Fragment>
          <div data-test="graph-chart" style={{ ...this.props.style, cursor: this.props.onBrushEnd ? 'ew-resize' : '' }}>
            <Chart renderer="canvas" className="story-chart">
              <Settings legendPosition={Position.Top} animateData={false} showLegendDisplayValue={false} onBrushEnd={this.props.onBrushEnd}/>
              <Axis id={getAxisId('bottom')} title={this.props.xLabel} position={Position.Bottom} showOverlappingTicks={true} tickFormat={this.getXTickFormatter()}/>
              <Axis id={getAxisId('left')} title={this.props.yLabel} position={Position.Left} tickFormat={this.props.type === 'bar' ? String : d => Number(d).toFixed(4)}/>
              {this.props.annotations &&
                this.getAnnotations()}
              {this.props.area && (!this.props.type || this.props.type === 'line') &&
                this.getAreaSeries()}
              {this.props.type === 'bar' ?
                this.getBarSeries(this.props.seriesLabels.length) :
                this.getLineSeries(this.props.seriesLabels.length)}
            </Chart>
          </div>
        </React.Fragment>);
        }
        else {
            return (<EuiFlexGroup justifyContent="spaceAround" alignItems="center" style={{ width: '100%', ...this.props.style }}>
          <EuiFlexItem grow={false}>
            <i style={{ fontStyle: 'italic' }}>No data found.</i>
          </EuiFlexItem>
        </EuiFlexGroup>);
        }
    }
}
