import React, { Fragment, PureComponent } from 'react';
import { JobService } from '../../services';
import { EuiButtonEmpty, EuiDescriptionList, EuiDescriptionListDescription, EuiDescriptionListTitle, EuiHealth, EuiInMemoryTable, EuiLoadingSpinner, EuiPageContentHeader, EuiPageContentHeaderSection, EuiTitle, RIGHT_ALIGNMENT } from '@elastic/eui';
import ReactJson from 'react-json-view';
import moment from 'moment';
import { prop, reverse, sortBy } from 'ramda';
import { setMlPageTitle } from '../../constants';
import { AppDependencies } from '../../AngularJsDependencies';
import queryString from 'query-string';
import './JobList.less';
export default class JobList extends PureComponent {
    constructor(props) {
        super(props);
        this.toggleDetails = (job) => {
            const itemIdToExpandedRowMap = { ...this.state.itemIdToExpandedRowMap };
            if (itemIdToExpandedRowMap[job.id]) {
                this.setState(prevState => {
                    delete itemIdToExpandedRowMap[job.id];
                    return {
                        itemIdToExpandedRowMap,
                        jobsWithInfoLoading: prevState.jobsWithInfoLoading.filter(id => id !== job.id)
                    };
                });
            }
            else {
                this.setState(prevState => ({ jobsWithInfoLoading: [...prevState.jobsWithInfoLoading, job.id] }));
                JobService.get(job.id)
                    .then(jobDetails => {
                    if (typeof jobDetails.result === 'object') {
                        return (<ReactJson src={jobDetails.result} name={false} enableClipboard={false} displayObjectSize={false} displayDataTypes={false}/>);
                    }
                    else {
                        return <div style={{ whiteSpace: 'pre-wrap', fontFamily: 'monospace', fontSize: '0.8rem' }}>{jobDetails.result}</div>;
                    }
                })
                    .then(node => this.setState(prevState => {
                    prevState.itemIdToExpandedRowMap[job.id] = node;
                    return {
                        itemIdToExpandedRowMap: prevState.itemIdToExpandedRowMap,
                        jobsWithInfoLoading: prevState.jobsWithInfoLoading.filter(id => id !== job.id)
                    };
                }));
            }
        };
        props.updateHeader({ currentTab: 'jobs' });
        this.selectedTask = queryString.parse(this.props.location.search).select;
        this.columns = [
            {
                name: 'Created',
                field: 'created',
                render: created => moment(created).format('YYYY-MM-DD HH:mm:ss')
            }, {
                name: 'Type',
                field: 'name'
            }, {
                name: 'Model',
                field: 'model'
            }, {
                name: 'Status',
                field: 'status',
                render: status => {
                    if (['Pending', 'Running'].includes(status)) {
                        return (<Fragment>
                <div style={{ width: '16px', marginRight: '2px', textAlign: 'center' }}>
                  <EuiLoadingSpinner size="s"/>
                </div>
                <span style={{ margin: '2px' }}>{status}</span>
              </Fragment>);
                    }
                    else {
                        return (<EuiHealth color={status === 'Complete' ? 'success' : 'danger'}>
                {status}
              </EuiHealth>);
                    }
                }
            }, {
                align: RIGHT_ALIGNMENT,
                width: '40px',
                isExpander: true,
                render: job => {
                    if (['Complete', 'Failed'].includes(job.status)) {
                        return (<EuiButtonEmpty onClick={() => this.toggleDetails(job)} isLoading={this.state.jobsWithInfoLoading.includes(job.id)} isDisabled={this.state.jobsWithInfoLoading.includes(job.id)} aria-label={this.state.itemIdToExpandedRowMap[job.id] ? 'Collapse' : 'Expand'} iconType={this.state.itemIdToExpandedRowMap[job.id] ? 'arrowUp' : 'arrowDown'}/>);
                    }
                }
            }
        ];
        this.state = {
            time: moment(),
            itemIdToExpandedRowMap: {},
            jobsWithInfoLoading: [],
            allowUpdate: true
        };
        setMlPageTitle('Jobs');
    }
    componentDidMount() {
        this.clockUpdateInterval = setInterval(() => this.setState({ time: moment() }), 1000);
        this.getJobs()
            .then(() => this.jobFetchInterval = setInterval(() => this.state.allowUpdate && this.getJobs().catch(() => clearInterval(this.jobFetchInterval)), 10000))
            .catch(() => undefined);
    }
    componentWillUnmount() {
        clearInterval(this.clockUpdateInterval);
        clearInterval(this.jobFetchInterval);
    }
    getJobs() {
        return JobService.getAll()
            .then(sortBy(prop('created')))
            .then(jobs => this.setState({ jobs: reverse(jobs) }))
            .catch(err => {
            this.context.notifier.error(`Could not get jobs: ${err.message}`);
            this.setState({ error: err.message });
            throw err;
        });
    }
    render() {
        return (<React.Fragment>
        <EuiPageContentHeader>
          <EuiPageContentHeaderSection>
            <EuiTitle><h2>Jobs</h2></EuiTitle>
          </EuiPageContentHeaderSection>
        </EuiPageContentHeader>

        <EuiDescriptionList style={{ padding: '16px 8px' }}>
          <EuiDescriptionListTitle style={{ fontSize: '14px' }}>Current time</EuiDescriptionListTitle>
          <EuiDescriptionListDescription>{this.state.time.format('YYYY-MM-DD HH:mm:ss')}</EuiDescriptionListDescription>
        </EuiDescriptionList>

        <EuiInMemoryTable items={this.state.jobs ? this.state.jobs : []} itemId="id" loading={!this.state.jobs && !this.state.error} 
        // @ts-ignore
        noItemsMessage={!this.state.jobs ? 'Loading jobs...' : 'No jobs found. Processes like model training jobs will be listed here.'} error={this.state.error} columns={this.columns} itemIdToExpandedRowMap={this.state.itemIdToExpandedRowMap} isExpandable={true} pagination={true} rowProps={(job) => ({ className: job.id === this.selectedTask ? 'temporary-highlight' : undefined })} onTableChange={({ page }) => this.setState({ allowUpdate: page.index === 0 })}/>
      </React.Fragment>);
    }
}
JobList.contextType = AppDependencies;
