import React, { Fragment } from 'react';
import queryString from 'query-string';
import moment from 'moment';
import { isEmpty, isEqual } from 'lodash';
import { EuiButton, EuiFieldText, EuiFlexGroup, EuiFlexItem, EuiForm, EuiFormRow, EuiIconTip, EuiLoadingSpinner, EuiPageContentHeader, EuiPageContentHeaderSection, EuiSuperDatePicker, EuiTitle } from '@elastic/eui';
import { ElasticsearchService, ModelService } from '../../services';
import { AggregationSelection, Graph, PredictionSettings } from '../../generic_components';
import { setMlPageTitle } from '../../constants';
import { getInterval, RelativeDateTranscoder } from '../../TimeUtils';
import { propEq } from 'ramda';
import { ScaleType } from '@elastic/charts';
import dateMath from '@elastic/datemath';
import { AppDependencies } from '../../AngularJsDependencies';
const MODEL_NAME_REGEX = /^[a-zA-Z0-9-_@ ()]+$/;
export default class EditModel extends React.PureComponent {
    constructor(props) {
        super(props);
        this.updateDatesInUrl = (startTime, endTime, { replaceHistory = false } = {}) => {
            const action = replaceHistory ? 'replace' : 'push';
            this.props.history[action]({
                pathname: this.props.location.pathname,
                search: '?' + queryString.stringify({
                    ...this.query,
                    startTime: RelativeDateTranscoder.encodeDate(startTime),
                    endTime: RelativeDateTranscoder.encodeDate(endTime)
                    // Encoding set to false as react-router pushes two locations onto the stack:
                    // one encoded, then one automatically decoded. This prevents backwards navigation
                }, { encode: false })
            });
        };
        this.isFullTimeRangeOnView = () => !!this.state.sourceInfo &&
            this.state.startTime === this.state.sourceInfo.dataStartTime &&
            this.state.endTime === this.state.sourceInfo.dataEndTime;
        this.modelCanBeCreated = () => {
            const genericModelCanBeCreated = !this.state.creationInProgress
                && this.state.data
                && !this.state.errorMessage
                && this.state.sourceInfo
                && this.state.searchParams
                && this.state.startTime
                && this.state.endTime
                && MODEL_NAME_REGEX.test(this.state.name);
            if (this.isPrediction) {
                return !!genericModelCanBeCreated && !!this.state.outputSize && this.state.outputSize % 1 === 0 && this.state.outputSize > 0;
            }
            else {
                return !!genericModelCanBeCreated;
            }
        };
        this.getData = ({ interval, index, query, timeField, aggType, field, startTime, endTime }) => {
            const parsedStartTime = dateMath.parse(startTime);
            const parsedEndTime = dateMath.parse(endTime, { roundUp: true });
            const actualBucketSize = getInterval(interval, parsedStartTime, parsedEndTime);
            this.setState({ errorMessage: undefined, actualBucketSize });
            return ElasticsearchService.dateHistogram(index, parsedStartTime, parsedEndTime, actualBucketSize, timeField, [{ field, aggType }], query)
                .then(data => this.setState({ data }))
                .catch(err => this.setState({ errorMessage: err.message }));
        };
        this.onModelSubmit = async () => {
            if (this.state.searchParams && this.state.sourceInfo && this.state.startTime && this.state.endTime) {
                this.setState({ creationInProgress: true });
                const startTime = dateMath.parse(this.state.startTime);
                const endTime = dateMath.parse(this.state.endTime, { roundUp: true });
                const modelSearchData = {
                    aggregationType: this.state.searchParams.aggregationType,
                    interval: this.state.searchParams.bucketSize,
                    field: this.state.searchParams.field
                };
                const model = {
                    type: this.isPrediction ? 'Prediction' : 'Anomaly',
                    name: this.state.name.trim(),
                    data: {
                        savedSearch: this.state.sourceInfo.savedSearchId,
                        index: this.state.sourceInfo.pattern,
                        aggregationType: modelSearchData.aggregationType,
                        interval: modelSearchData.interval,
                        timeField: this.state.sourceInfo.timeField,
                        start: startTime.toISOString(),
                        end: endTime.toISOString(),
                        filters: {
                            must: this.state.sourceInfo.query.query.bool.must,
                            mustNot: this.state.sourceInfo.query.query.bool.must_not
                        }
                    },
                    inputs: modelSearchData.aggregationType === 'count' ? [] : [modelSearchData.field],
                    inputSize: 30
                };
                if (this.isPrediction) {
                    model.outputSize = this.state.outputSize;
                }
                if (this.props.match && this.props.match.params.id) {
                    const isUpdate = this.state.previousTrainingStatus !== 'failed' &&
                        isEqual(this.state.originalSearchParams, modelSearchData) &&
                        (!this.isPrediction || (this.isPrediction && this.state.previousOutputSize === this.state.outputSize));
                    ModelService.update(model, isUpdate)
                        .then(() => this.props.history.push(`/models`))
                        .catch(err => {
                        this.setState({ creationInProgress: false });
                        this.context.notifier.error('Error updating model: ' + err.message);
                    });
                }
                else {
                    ModelService.create(model)
                        .then(() => this.props.history.push(`/models`))
                        .catch(err => {
                        this.setState({ creationInProgress: false });
                        this.context.notifier.error('Error creating model: ' + err.message);
                    });
                }
            }
        };
        this.getFullTimeRange = (pattern, timeField) => {
            return ElasticsearchService.statField(pattern, timeField)
                .then(stats => [moment(stats.min), moment(stats.max)])
                .then(([min, max]) => min.isSame(max) ? [min.subtract(1, 's'), max.add(1, 's')] : [min, max])
                .then(([min, max]) => [min.toISOString(), max.toISOString()])
                .catch(err => {
                this.context.notifier.error(`Error finding full time range of ${pattern}: ` + err.message);
                return ['now-15y', 'now'];
            });
        };
        this.handleSelectionBrushEnd = (startTime, endTime) => this.updateDatesInUrl(moment(startTime).toISOString(), moment(endTime).toISOString());
        this.query = queryString.parse(this.props.location.search);
        this.isPrediction = this.props.location.pathname.includes('prediction');
        this.state = {
            creationInProgress: false,
            graphIsValid: false,
            outputSize: 10,
            name: props.match.params.id || '',
            originalSearchParams: {},
            startTime: this.query.startTime ? RelativeDateTranscoder.decodeDate(this.query.startTime) : undefined,
            endTime: this.query.endTime ? RelativeDateTranscoder.decodeDate(this.query.endTime) : undefined
        };
        this.updateDatePicker(this.state.startTime, this.state.endTime);
        this.historyUnlisten = this.props.history.listen(location => {
            let { startTime, endTime } = queryString.parse(location.search);
            if (startTime && endTime) {
                startTime = RelativeDateTranscoder.decodeDate(startTime);
                endTime = RelativeDateTranscoder.decodeDate(endTime);
                this.setState({ startTime, endTime });
                this.updateDatePicker(startTime, endTime);
                if (this.state.searchParams && this.state.sourceInfo) {
                    this.getData({
                        interval: this.state.searchParams.bucketSize,
                        index: this.state.sourceInfo.pattern,
                        query: this.state.sourceInfo.query,
                        timeField: this.state.sourceInfo.timeField,
                        aggType: this.state.searchParams.aggregationType,
                        field: this.state.searchParams.field,
                        startTime,
                        endTime
                    });
                }
            }
        });
        setMlPageTitle();
    }
    componentDidMount() {
        if (this.props.match && this.props.match.params.id) {
            this.getModelSearchParams()
                .catch(err => this.context.notifier.error(`Could not get model ${this.props.match.params.id}: ${err.message}`));
        }
        else {
            this.getSavedSearchInfo(this.query.savedSearch)
                .then(sourceInfo => {
                setMlPageTitle(`Model for ${sourceInfo && sourceInfo.savedSearchTitle}`);
                this.setState({ sourceInfo });
            })
                .catch(err => this.context.notifier.error(`Failed getting search info: ${err.message}`));
        }
    }
    updateDatePicker(startTime, endTime) {
        if (startTime && endTime) {
            this.props.updateHeader({
                headerActions: (<EuiSuperDatePicker start={startTime || ''} end={endTime || ''} onTimeChange={props => props.isInvalid || this.updateDatesInUrl(props.start, props.end)} showUpdateButton={false} dateFormat="YYYY-MM-DD HH:mm:ss"/>)
            });
        }
        else {
            this.props.updateHeader({
                headerActions: <div style={{ width: '150px', display: 'flex', alignItems: 'center' }}><EuiLoadingSpinner size="l"/></div>
            });
        }
    }
    async getModelSearchParams() {
        const model = await ModelService.get(this.props.match.params.id);
        setMlPageTitle(`Edit ${model.name}`);
        this.setState({
            sourceInfo: await this.getSavedSearchInfo(model.data.savedSearch),
            searchParams: {
                aggregationType: model.data.aggregationType,
                field: model.inputs[0],
                bucketSize: model.data.interval
            },
            originalSearchParams: {
                aggregationType: model.data.aggregationType,
                interval: model.data.interval,
                field: model.inputs[0]
            },
            startTime: model.data.start,
            endTime: model.data.end,
            outputSize: model.outputSize,
            name: model.name,
            previousOutputSize: model.outputSize,
            previousTrainingStatus: model.training.state
        });
        this.updateDatesInUrl(model.data.start, model.data.end, { replaceHistory: true });
    }
    async getSavedSearchInfo(savedSearchId) {
        const savedSearch = await this.context.savedObjectsClient.get('search', savedSearchId);
        if (savedSearch.error) {
            throw new Error(`Error retrieving saved search'${savedSearch.id}': ${savedSearch.error.reason}`);
        }
        const metaData = JSON.parse(savedSearch.attributes.kibanaSavedObjectMeta.searchSourceJSON);
        const indexPattern = await this.context.savedObjectsClient.get('index-pattern', metaData.index);
        if (indexPattern.error) {
            throw new Error(`Error retrieving index pattern'${indexPattern.id}': ${indexPattern.error.reason}`);
        }
        const filters = metaData.filter || [];
        const queries = metaData.query ? [metaData.query] : [];
        const query = this.context.queryBuilder(filters, queries);
        const [dataStartTime, dataEndTime] = await this.getFullTimeRange(indexPattern.attributes.title, indexPattern.attributes.timeFieldName);
        const savedSearchInfo = {
            savedSearchTitle: savedSearch.attributes.title,
            savedSearchId: savedSearchId,
            pattern: indexPattern.attributes.title,
            timeField: indexPattern.attributes.timeFieldName,
            fields: JSON.parse(indexPattern.attributes.fields),
            query,
            dataStartTime,
            dataEndTime
        };
        if (!this.state.startTime || !this.state.endTime) {
            this.updateDatesInUrl(dataStartTime, dataEndTime, { replaceHistory: true });
        }
        return savedSearchInfo;
    }
    render() {
        return (<Fragment>
        <EuiPageContentHeader>
          <EuiPageContentHeaderSection>
            <EuiTitle data-test="model-page-header">
              {this.props.match && this.props.match.params.id ?
            <h1>Edit {this.state.name}</h1> :
            <h1>Model for {this.state.sourceInfo && this.state.sourceInfo.savedSearchTitle}</h1>}
            </EuiTitle>
          </EuiPageContentHeaderSection>
          <EuiPageContentHeaderSection>
            {this.state.model && this.state.actualBucketSize !== this.state.model.data.interval &&
            <EuiIconTip size="l" type="alert" color="warning" content={`Bucket size is too small to display so ${this.state.actualBucketSize} was used instead.
                        ${this.state.model.data.interval} will still be used in detections.`}/>}
            <EuiButton isDisabled={this.isFullTimeRangeOnView()} onClick={() => {
            if (this.state.sourceInfo) {
                this.updateDatesInUrl(this.state.sourceInfo.dataStartTime, this.state.sourceInfo.dataEndTime);
            }
        }} data-test="find-full-range-btn">
              Find full range
            </EuiButton>
          </EuiPageContentHeaderSection>
        </EuiPageContentHeader>
        {this.state.sourceInfo && this.state.sourceInfo.fields &&
            <EuiFlexGroup>
            <EuiFlexItem style={{ flex: '0 0 300px' }}>
              <AggregationSelection onUpdate={searchParams => this.onSearchParamsUpdate(searchParams, this.state.sourceInfo)} aggregationType={this.state.searchParams ? this.state.searchParams.aggregationType : undefined} field={this.state.searchParams ? this.state.searchParams.field : undefined} bucketSize={this.state.searchParams ? this.state.searchParams.bucketSize : ''} actualBucketSize={this.state.actualBucketSize} fields={this.state.sourceInfo.fields
                .filter(propEq('type', 'number'))
                .map(field => field.name)}/>
              {this.isPrediction &&
                <div style={{ marginTop: 20 }}>
                  <PredictionSettings bucketCount={this.state.outputSize} onSizeChange={outputSize => this.setState({ outputSize })} interval={this.state.searchParams && this.state.searchParams.bucketSize}/>
                </div>}
              {this.renderModelInfoInputs()}
            </EuiFlexItem>
            {isEmpty(this.state.searchParams) ?
                <EuiFlexGroup justifyContent="spaceAround" alignItems="center" style={{ height: '400px', width: '100%' }}>
                <EuiFlexItem grow={false}>
                  <i style={{ fontStyle: 'italic' }}>Fill in the aggregation parameters on the left to see a preview.</i>
                </EuiFlexItem>
              </EuiFlexGroup> :
                <EuiFlexItem>
                <Graph isLoading={!this.state.data} errorMessage={this.state.errorMessage} style={{ height: '100%' }} xLabel={this.state.sourceInfo.timeField} yLabel={this.state.searchParams ? this.state.searchParams.field : ''} xLabelType={ScaleType.Time} seriesLabels={this.state.searchParams ?
                    [this.state.searchParams.aggregationType === 'count' ? 'count' : this.state.searchParams.field] : []} data={this.state.data} onBrushEnd={this.handleSelectionBrushEnd}/>
              </EuiFlexItem>}
          </EuiFlexGroup>}
      </Fragment>);
    }
    onSearchParamsUpdate(searchParams, sourceInfo) {
        if (sourceInfo && this.state.startTime && this.state.endTime) {
            this.setState({ searchParams });
            this.getData({
                interval: searchParams.bucketSize,
                index: sourceInfo.pattern,
                query: sourceInfo.query,
                timeField: sourceInfo.timeField,
                aggType: searchParams.aggregationType,
                field: searchParams.field,
                startTime: this.state.startTime,
                endTime: this.state.endTime
            });
        }
    }
    renderModelInfoInputs() {
        return (<EuiForm style={{ marginTop: 17 }}>
        <EuiFormRow label="Name" fullWidth isInvalid={!!this.state.name && !MODEL_NAME_REGEX.test(this.state.name)} error="Name must must only contain alphanumerics, spaces and characters @_-()">
          <EuiFieldText data-test="new-model-name" name="model-name" value={this.state.name} onChange={e => this.setState({ name: e.target.value })} disabled={!!(this.props.match && this.props.match.params.id)}/>
        </EuiFormRow>
        <EuiFormRow style={{ marginTop: 20 }}>
          <EuiButton data-test="create-retrain-btn" fill isDisabled={!this.modelCanBeCreated()} onClick={this.onModelSubmit}>
            <EuiFlexGroup>
              <EuiFlexItem>
                {(() => {
            if (this.state.creationInProgress) {
                return (<EuiLoadingSpinner size="l"/>);
            }
            else if (this.props.match && this.props.match.params.id) {
                return 'Retrain model';
            }
            else {
                return 'Create model';
            }
        })()}
              </EuiFlexItem>
            </EuiFlexGroup>
          </EuiButton>
        </EuiFormRow>
      </EuiForm>);
    }
}
EditModel.contextType = AppDependencies;
