import React, { Fragment } from 'react';
import queryString from 'query-string';
import moment from 'moment';
import { EuiButton, EuiFlexGroup, EuiFlexItem, EuiIconTip, EuiLoadingSpinner, EuiPageContentHeader, EuiPageContentHeaderSection, EuiSpacer, EuiSuperDatePicker, EuiTitle } from '@elastic/eui';
import { ScaleType } from '@elastic/charts';
import { ElasticsearchService, ModelService } from '../../services';
import { Graph } from '../../generic_components';
import { getInterval, RelativeDateTranscoder } from '../../TimeUtils';
import { setMlPageTitle } from '../../constants';
import dateMath from '@elastic/datemath';
import { AppDependencies } from '../../AngularJsDependencies';
export default class HistoricalDetection extends React.PureComponent {
    constructor(props) {
        super(props);
        this.updateDates = (startTime, endTime, options) => {
            this.updateDatePicker(startTime, endTime);
            this.updateDatesInUrl(startTime, endTime, options);
        };
        this.updateDatesInUrl = (startTime, endTime, { replaceHistory = false } = {}) => {
            const action = replaceHistory ? 'replace' : 'push';
            this.props.history[action]({
                pathname: this.props.location.pathname,
                search: '?' + queryString.stringify({
                    ...this.query,
                    startTime: RelativeDateTranscoder.encodeDate(startTime),
                    endTime: RelativeDateTranscoder.encodeDate(endTime)
                    // Encoding set to false as react-router pushes two locations onto the stack:
                    // one encoded, then one automatically decoded. This prevents backwards navigation
                }, { encode: false })
            });
        };
        this.setFullTimeRange = (model) => {
            ElasticsearchService.statField(model.data.index, model.data.timeField)
                .then(stats => [moment(stats.min), moment(stats.max)])
                .then(([min, max]) => min.isSame(max) ? [min.subtract(1, 's'), max.add(1, 's')] : [min, max])
                .then(([min, max]) => this.updateDates(min.toISOString(), max.toISOString()))
                .catch(err => this.context.notifier.error(`Error finding full time range of ${model.data.index}:  ${err.message}`));
        };
        this.onDetectionSubmit = () => {
            this.setState({ sendingDetectionRequest: true });
            const startTime = dateMath.parse(this.state.startTime);
            const endTime = dateMath.parse(this.state.endTime, { roundUp: true });
            ModelService.predict(this.props.match.params.id, startTime, endTime)
                .then(taskId => {
                this.context.notifier.info(`Detection now in progress.`);
                this.props.history.push(`/jobs?select=${taskId}`);
            })
                .catch(err => {
                this.setState({ sendingDetectionRequest: false });
                this.context.notifier.error('Error starting detection: ' + err.message);
            });
        };
        this.getData = (model, startTime, endTime) => {
            const parsedStartTime = dateMath.parse(startTime);
            const parsedEndTime = dateMath.parse(endTime, { roundUp: true });
            const actualBucketSize = getInterval(model.data.interval, parsedStartTime, parsedEndTime);
            this.setState({
                graphIsValid: false,
                isLoading: true,
                errorMessage: undefined,
                actualBucketSize
            });
            return ElasticsearchService.dateHistogram(model.data.index, parsedStartTime, parsedEndTime, actualBucketSize, model.data.timeField, [{ field: model.inputs[0], aggType: model.data.aggregationType }], {
                query: {
                    bool: {
                        must: model.data.filters.must,
                        must_not: model.data.filters.mustNot
                    }
                }
            })
                .then(data => this.setState({
                graphIsValid: true,
                isLoading: false,
                data
            }))
                .catch(err => this.setState({
                isLoading: false,
                errorMessage: err.message
            }));
        };
        this.handleSelectionBrushEnd = (startTime, endTime) => this.updateDates(moment(startTime).toISOString(), moment(endTime).toISOString());
        this.query = queryString.parse(this.props.location.search);
        this.state = {
            sendingDetectionRequest: false,
            graphIsValid: false,
            isLoading: true,
            data: [],
            startTime: this.query.startTime ? RelativeDateTranscoder.decodeDate(this.query.startTime) : 'now-15y',
            endTime: this.query.endTime ? RelativeDateTranscoder.decodeDate(this.query.endTime) : 'now'
        };
        this.updateDatePicker(this.state.startTime, this.state.endTime);
        this.historyUnlisten = this.props.history.listen(location => {
            let { startTime, endTime } = queryString.parse(location.search);
            if (startTime && endTime) {
                startTime = RelativeDateTranscoder.decodeDate(startTime);
                endTime = RelativeDateTranscoder.decodeDate(endTime);
                this.setState({ startTime, endTime });
                this.updateDatePicker(startTime, endTime);
                if (this.state.model) {
                    this.getData(this.state.model, startTime, endTime);
                }
            }
        });
        setMlPageTitle();
    }
    componentDidMount() {
        this.getSearchParams()
            .then(model => {
            setMlPageTitle(`Historical Detection for ${model.name}`);
            this.getData(model, this.state.startTime, this.state.endTime);
        });
    }
    componentWillUnmount() {
        this.historyUnlisten();
    }
    updateDatePicker(startTime, endTime) {
        this.props.updateHeader({
            headerActions: (<EuiSuperDatePicker isLoading={!startTime || !endTime} start={startTime || ''} end={endTime || ''} onTimeChange={props => props.isInvalid || this.updateDatesInUrl(props.start, props.end)} showUpdateButton={false} dateFormat="YYYY-MM-DD HH:mm:ss"/>)
        });
    }
    getSearchParams() {
        return ModelService.get(this.props.match.params.id)
            .then(async (model) => {
            this.setState({ model });
            await this.setFullTimeRange(model);
            return model;
        })
            .catch(err => {
            this.context.notifier.error(`Could not get model ${this.props.match.params.id}: ${err.message}`);
            throw err;
        });
    }
    render() {
        return (<Fragment>
        <EuiPageContentHeader>
          <EuiPageContentHeaderSection>
            <EuiTitle><h1>Historical Detection for {this.props.match.params.id}</h1></EuiTitle>
          </EuiPageContentHeaderSection>
          <EuiPageContentHeaderSection>
            {this.state.model && this.state.actualBucketSize !== this.state.model.data.interval &&
            <EuiIconTip size="l" type="alert" color="warning" content={`Bucket size is too small to display so ${this.state.actualBucketSize} was used instead.
                    ${this.state.model.data.interval} will still be used in detections.`}/>}
            <EuiButton aria-label="findFullRange" style={{ marginLeft: '10px' }} isDisabled={!this.state.model} onClick={() => this.state.model && this.setFullTimeRange(this.state.model)}>
              Find full range
            </EuiButton>
          </EuiPageContentHeaderSection>
        </EuiPageContentHeader>
        <EuiSpacer size="l"/>

        <Graph isLoading={this.state.isLoading || !this.state.model} errorMessage={this.state.errorMessage} style={{ height: '500px' }} xLabel={this.state.model ? this.state.model.data.timeField : ''} yLabel={this.state.model ? this.state.model.inputs[0] : ''} xLabelType={ScaleType.Time} seriesLabels={this.state.model ?
            [this.state.model.data.aggregationType === 'count' ? 'count' : this.state.model.inputs[0]] : []} data={this.state.data} onBrushEnd={this.handleSelectionBrushEnd}/>

        <EuiButton fill aria-label="predict" isDisabled={this.state.sendingDetectionRequest || !this.state.graphIsValid || !this.state.model} onClick={this.onDetectionSubmit} style={{ marginTop: '20px' }}>
          <EuiFlexGroup>
            <EuiFlexItem>
              {this.state.sendingDetectionRequest ? <EuiLoadingSpinner size="l"/> : 'Detect'}
            </EuiFlexItem>
          </EuiFlexGroup>
        </EuiButton>
      </Fragment>);
    }
}
HistoricalDetection.contextType = AppDependencies;
