import React, { Fragment } from 'react';
import moment from 'moment';
import { EuiButtonEmpty, EuiContextMenuItem, EuiContextMenuPanel, EuiDescriptionList, EuiFlexGroup, EuiFlexItem, EuiIconTip, EuiLoadingSpinner, EuiPageContentHeader, EuiPageContentHeaderSection, EuiPopover, EuiText, EuiTitle, EuiPanel, EuiSpacer, EuiCallOut } from '@elastic/eui';
import { CircularProgressbarWithChildren, buildStyles } from 'react-circular-progressbar';
import 'react-circular-progressbar/dist/styles.css';
import { Graph } from '../../generic_components';
import { ElasticsearchService, ModelService } from '../../services';
import './ModelInfo.less';
import { sirenMlIndex, trainingInProgressStatuses, COLOURS, setMlPageTitle } from '../../constants';
// eslint-disable-next-line import/no-unresolved
import { getInterval } from '../../TimeUtils';
import { ScaleType } from '@elastic/charts';
import { isEqual } from 'lodash';
import { AppDependencies } from '../../AngularJsDependencies';
import { RadialSeparators } from './RadialSeparators';
export default class ModelInfo extends React.PureComponent {
    constructor(props) {
        super(props);
        this.setTrainingRmse = (modelId) => {
            ElasticsearchService.query(sirenMlIndex, [
                { field: 'model_id.keyword', value: modelId },
                { field: 'phase.keyword', value: 'test' },
                { field: 'step.keyword', value: '0' }
            ])
                .then(result => this.setState({ rmse: result[0] && result[0].rmse }));
        };
        this.toggleActionMenu = () => this.setState(prevState => ({ showActionMenu: !prevState.showActionMenu }));
        this.toggleStart = () => {
            if (this.state.model) {
                this.setState({ actionLoading: true, showActionMenu: false });
                const action = this.state.model.active ? ModelService.stop : ModelService.start;
                action(this.state.model.name)
                    .then(() => this.getModelInfo())
                    .catch(err => this.context.notifier.error(`Could not ${action.name} model: ${err.message}`))
                    .finally(() => this.setState({ actionLoading: false }));
            }
        };
        this.deleteModel = () => {
            this.setState({ actionLoading: true, showActionMenu: false });
            this.state.model && ModelService.delete(this.state.model.name)
                .then(() => this.props.history.push(`/models`))
                .catch(err => this.context.notifier.error(`Could not delete model: ${err.message}`))
                .finally(() => this.setState({ actionLoading: false }));
        };
        props.updateHeader({});
        this.state = {
            model: undefined,
            showActionMenu: false,
            actionLoading: false,
            trainingData: []
        };
        setMlPageTitle();
    }
    componentDidMount() {
        this.getModelInfo()
            .then(model => {
            setMlPageTitle(`${model.name} Info`);
            if (trainingInProgressStatuses.includes(model.training.state)) {
                this.setIntervalRef = setInterval(() => {
                    this.getModelInfo()
                        .then(model => {
                        if (!trainingInProgressStatuses.includes(model.training.state)) {
                            clearInterval(this.setIntervalRef);
                            if (model.training.state === 'trained') {
                                this.setTrainingRmse(model.id);
                            }
                        }
                    });
                }, 5000);
            }
            else if (model.training.state === 'trained') {
                this.setTrainingRmse(model.id);
            }
        });
    }
    componentWillUnmount() {
        clearInterval(this.setIntervalRef);
    }
    getModelInfo() {
        return ModelService.get(this.props.match.params.id)
            .then(model => {
            this.setState({ model });
            return model;
        });
    }
    render() {
        return (<Fragment>
        <EuiPageContentHeader>
          <EuiPageContentHeaderSection>
            <EuiTitle data-test="model-title"><h2>{this.props.match.params.id}</h2></EuiTitle>
          </EuiPageContentHeaderSection>
          <EuiPageContentHeaderSection>
            {this.renderActionButton()}
          </EuiPageContentHeaderSection>
        </EuiPageContentHeader>
        {this.getBody()}
      </Fragment>);
    }
    renderActionButton() {
        if (this.state.model) {
            const items = [
                <EuiContextMenuItem key="0" disabled={this.state.model.training.state !== 'trained'} icon={this.state.model.active ? 'stop' : 'play'} onClick={this.toggleStart} toolTipPosition="left" toolTipContent={`${this.state.model.active ? 'Stop' : 'Start'} live 
                  ${this.state.model.type === 'Anomaly' ? 'detection of anomalies' : 'prediction'}`}>
          {this.state.model.active ? 'Deactivate' : 'Activate'}
        </EuiContextMenuItem>,
                <EuiContextMenuItem key="2" icon="pencil" disabled={!['trained', 'failed'].includes(this.state.model.training.state)} href={this.state.model.type === 'Prediction' ?
                    `#/model/prediction/${this.props.match.params.id}/edit` :
                    `#/model/detection/${this.props.match.params.id}/edit`}>
          Edit
        </EuiContextMenuItem>,
                <EuiContextMenuItem key="3" icon="trash" onClick={this.deleteModel}>Delete</EuiContextMenuItem>
            ];
            if (this.state.model.type === 'Anomaly') {
                items.splice(1, 0, <EuiContextMenuItem key="1" icon="clock" disabled={this.state.model.training.state !== 'trained'} href={`#/model/${this.props.match.params.id}/historicalDetection`} toolTipPosition="left" toolTipContent="Detect anomalies from a past time range">
            Apply historically
          </EuiContextMenuItem>);
            }
            return (<EuiPopover id="contextMenu" button={<EuiButtonEmpty isLoading={this.state.actionLoading} iconType="boxesVertical" onClick={this.toggleActionMenu}/>} isOpen={this.state.showActionMenu} closePopover={this.toggleActionMenu} anchorPosition="downRight">
          <EuiContextMenuPanel items={items}/>
        </EuiPopover>);
        }
    }
    getBody() {
        if (this.state.model) {
            return (<Fragment>
          {this.state.model.training.error ?
                <div>
              <EuiCallOut title="An error occurred during training!" color="danger" iconType="alert">
                <p>{this.state.model.training.error}</p>
              </EuiCallOut>
              <EuiSpacer size="m"/>
            </div>
                :
                    undefined}
          <EuiFlexGroup>
            <EuiFlexItem grow={2}>
              <EuiPanel paddingSize="m" hasShadow>
                <ModelStatus model={this.state.model}/>
              </EuiPanel>
            </EuiFlexItem>
            <EuiFlexItem grow={3}>
              <EuiPanel paddingSize="m" hasShadow>
                <TrainingConfiguration model={this.state.model} rmse={this.state.rmse}/>
              </EuiPanel>
            </EuiFlexItem>
          </EuiFlexGroup>
          <EuiSpacer size="m"/>
          <EuiPanel paddingSize="m" hasShadow>
            <TrainingConfigurationGraph model={this.state.model}/>
          </EuiPanel>
          <EuiSpacer size="m"/>
          <EuiPanel paddingSize="m" hasShadow>
            <TrainingGraphs model={this.state.model}/>
          </EuiPanel>
        </Fragment>);
        }
        else {
            return (<EuiFlexGroup justifyContent="spaceAround" alignItems="center">
          <EuiFlexItem grow={false}>
            <EuiLoadingSpinner size="xl"/>
          </EuiFlexItem>
        </EuiFlexGroup>);
        }
    }
}
ModelInfo.contextType = AppDependencies;
const SectionHeader = (props) => {
    return (<EuiText style={{ marginTop: '10px', marginBottom: '10px' }}>
      <h3>{props.children}</h3>
    </EuiText>);
};
const ModelStatus = (props) => {
    const progress = trainingInProgressStatuses.includes(props.model.training.state) ? props.model.training.progress : 100;
    const state = props.model.active ? 'active' : props.model.training.state;
    const getProgressCircleColor = (progressStatus) => {
        if (props.model.active) {
            return '#DD0A73'; // pink eui-accent
        }
        switch (progressStatus) {
            case 'trained':
                return '#017D73'; //green eui-secondary
            case 'failed':
                return '#BD271E'; // red eui-danger
            default:
                return '#006BB4'; // blue eui-primary
        }
    };
    const progressBarChild = () => {
        return (<div style={{ padding: '100px' }}>
        {!['pending', 'trained', 'failed'].includes(props.model.training.state) ?
            <EuiText textAlign="center" grow={false}>
            <h2>
              {progress !== undefined ? +progress : 0}%
            </h2>
          </EuiText>
            :
                undefined}
        <EuiText textAlign="center" color="subdued">
          {['pending', 'trained', 'failed'].includes(props.model.training.state) ?
            <h2 data-test="model-status" style={{ textTransform: 'capitalize' }}>
              {state}
            </h2>
            :
                <p data-test="model-status" style={{ textTransform: 'capitalize' }}>
              {state}
            </p>}
        </EuiText>
      </div>);
    };
    return (<div style={{ maxHeight: '300px', maxWidth: '300px', margin: 'auto', padding: '20px' }}>
      <CircularProgressbarWithChildren value={progress !== undefined ? +progress : 0} styles={buildStyles({
        pathColor: getProgressCircleColor(props.model.training.state),
        strokeLinecap: 'butt'
    })}>
        {props.model.active ?
        <div style={{ animation: 'spin 30s linear infinite', height: '100%', width: '100%', position: 'absolute' }}>
            <RadialSeparators count={12} style={{
            margin: 'auto',
            background: '#fff',
            width: '10%',
            height: `20%` // This needs to be equal to props.strokeWidth
        }}/>
          </div>
        :
            undefined}
        {progressBarChild({})}
      </CircularProgressbarWithChildren>
    </div>);
};
const TrainingConfiguration = ({ model, rmse }) => {
    const rsmeInfo = () => {
        return (<Fragment>
        <EuiIconTip content="
            Root Mean Square Error is a measure of how far away the predicted values are from the real values.
            Lower is better. It should be within 5% of the typical span of your data.
          " position="right"/>
        <p>{rmse}</p>
      </Fragment>);
    };
    const firstColumn = [
        { title: 'Type', description: model.type },
        { title: 'Index', description: model.data.index },
        { title: 'Aggregation type', description: model.data.aggregationType },
        { title: 'Time field', description: model.data.timeField },
        { title: 'Inputs', description: model.inputs.join(', ') },
        { title: 'Interval', description: model.data.interval },
        { title: 'Start', description: moment(model.data.start).format('YYYY-MM-DD HH:mm:ss Z') },
        { title: 'End', description: moment(model.data.end).format('YYYY-MM-DD HH:mm:ss Z') },
    ];
    if (rmse) {
        firstColumn.push({ title: 'RMSE', description: rsmeInfo() });
    }
    return (<Fragment>
      <SectionHeader>Training Configuration</SectionHeader>
      <div style={{ display: 'flex', alignItems: 'flex-start' }}>
        <EuiDescriptionList type="column" listItems={firstColumn} titleProps={{ className: 'description-title' }} descriptionProps={{ className: 'description-body' }}/>
      </div>
    </Fragment>);
};
class TrainingConfigurationGraph extends React.PureComponent {
    constructor(props) {
        super(props);
        this.state = {};
    }
    componentDidMount() {
        this.getData(this.props.model);
    }
    componentDidUpdate(prevProps) {
        if (!isEqual(this.props.model.data, prevProps.model.data)) {
            this.getData(this.props.model);
        }
    }
    getData(model) {
        const momentToStart = moment(model.data.start);
        const momentToEnd = moment(model.data.end);
        const actualBucketSize = getInterval(model.data.interval, momentToStart, momentToEnd);
        return ElasticsearchService.dateHistogram(model.data.index, momentToStart, momentToEnd, actualBucketSize, model.data.timeField, [{ field: model.inputs[0], aggType: model.data.aggregationType }], {
            query: {
                bool: {
                    must: model.data.filters.must,
                    must_not: model.data.filters.mustNot
                }
            }
        })
            .then(data => this.setState({ data }))
            .catch(err => this.setState({ errorMessage: err.message }));
    }
    render() {
        const model = this.props.model;
        return (<Fragment>
        <div>
          <Graph isLoading={!this.state.data} errorMessage={this.state.errorMessage} style={{ height: '300px', padding: '10px' }} xLabel={model ? model.data.timeField : ''} yLabel={model ? model.inputs[0] : ''} xLabelType={ScaleType.Time} seriesLabels={[model.data.aggregationType === 'count' ? 'count' : model.inputs[0]]} data={this.state.data}/>
        </div>
      </Fragment>);
    }
}
class TrainingGraphs extends React.PureComponent {
    constructor(props) {
        super(props);
        this.getHyperOptData = () => {
            this.setState(prevState => ({
                hyperOpt: {
                    ...prevState.hyperOpt,
                    errorMessage: undefined
                }
            }));
            const searchQuery = {
                query: {
                    bool: {
                        must: [
                            { match: { 'model_id.keyword': this.props.model.id } },
                            { match: { 'phase.keyword': 'hpopt' } }
                        ]
                    }
                }
            };
            return ElasticsearchService.histogram(sirenMlIndex, 'hpopt_trial', 1, 'avg', ['hpopt_metric'], searchQuery)
                .then(data => this.setState({ hyperOpt: { data } }))
                .catch(err => {
                this.setState(prevState => ({
                    hyperOpt: {
                        ...prevState.hyperOpt,
                        errorMessage: err.message
                    }
                }));
                throw err;
            });
        };
        this.getTrainingData = () => {
            this.setState(prevState => ({
                training: {
                    ...prevState.training,
                    errorMessage: undefined
                }
            }));
            const searchQuery = {
                query: {
                    bool: {
                        must: [
                            { match: { 'model_id.keyword': this.props.model.id } },
                            { match: { 'phase.keyword': 'train' } }
                        ]
                    }
                }
            };
            return ElasticsearchService.histogram(sirenMlIndex, 'epoch_end', 1, 'avg', ['loss', 'val_loss'], searchQuery)
                .then(data => this.setState({ training: { data } }))
                .catch(err => {
                this.setState(prevState => ({
                    training: {
                        ...prevState.training,
                        errorMessage: err.message
                    }
                }));
                throw err;
            });
        };
        this.state = {
            hyperOpt: {},
            training: {}
        };
    }
    componentDidMount() {
        Promise.all([
            this.getHyperOptData(),
            this.getTrainingData()
        ])
            .then(() => {
            if (this.props.model && trainingInProgressStatuses.includes(this.props.model.training.state)) {
                this.setIntervalRef = setInterval(() => {
                    if (this.props.model && !trainingInProgressStatuses.includes(this.props.model.training.state)) {
                        clearInterval(this.setIntervalRef);
                    }
                    else {
                        this.getHyperOptData();
                        this.getTrainingData();
                    }
                }, 5000);
            }
        })
            .catch(() => clearInterval(this.setIntervalRef));
    }
    componentWillUnmount() {
        clearInterval(this.setIntervalRef);
    }
    render() {
        return (<div>
        <SectionHeader>Training Progress</SectionHeader>
        <div style={{ display: 'flex', height: '300px' }}>
          <div style={{ flexGrow: 1, display: 'flex', flexBasis: '0', flexDirection: 'column', margin: '20px' }}>
            <b style={{ marginBottom: '10px' }}>Hyperparameter Optimisation</b>
            <Graph isLoading={!this.state.hyperOpt.data} errorMessage={this.state.hyperOpt.errorMessage} style={{ height: '100%' }} type="bar" xLabel="Trial" yLabel="Loss" seriesLabels={['Loss']} data={this.state.hyperOpt.data}/>
          </div>
          <div style={{ flexGrow: 1, display: 'flex', flexBasis: '0', flexDirection: 'column', margin: '20px' }}>
            <b style={{ marginBottom: '10px' }}>Training Loss</b>
            <Graph isLoading={!this.state.training.data} errorMessage={this.state.training.errorMessage} style={{ height: '100%' }} xLabel="Trial" yLabel="Loss" xTickFormatter={String} seriesLabels={['Loss', 'Validation loss']} lineColours={[COLOURS.blue, COLOURS.yellow]} data={this.state.training.data}/>
          </div>
        </div>
      </div>);
    }
}
