import React from 'react';
import { cloneDeep, omit, sortBy } from 'lodash';
import { ModelService } from '../../services';
import { EuiBasicTable, EuiButtonIcon, EuiFlexGroup, EuiFlexItem, EuiHeaderLink, EuiLink, EuiLoadingSpinner, EuiPageContentHeader, EuiPageContentHeaderSection, EuiProgress, EuiSwitch, EuiTitle, EuiToolTip } from '@elastic/eui';
import { setMlPageTitle, trainingInProgressStatuses } from '../../constants';
import moment from 'moment';
import { AppDependencies } from '../../AngularJsDependencies';
export default class ModelList extends React.PureComponent {
    constructor(props) {
        super(props);
        this.deleteModel = (modelName) => {
            this.setState(prevState => ({ deletingInProgressMap: { ...prevState.deletingInProgressMap, [modelName]: true } }));
            ModelService.delete(modelName)
                .catch(err => this.context.notifier.error(`Could not delete model: ${err.message}`))
                .finally(() => this.getModels()
                .then(() => this.setState(prevState => ({ deletingInProgressMap: omit(prevState.deletingInProgressMap, modelName) }))));
        };
        this.toggleStart = (model) => {
            this.setState(prevState => {
                const state = cloneDeep(prevState);
                if (state.models) {
                    state.models.find(m => m.name === model.name).active = !model.active;
                    state.deploymentChangingMap[model.name] = true;
                    return state;
                }
                else {
                    return {};
                }
            });
            const action = model.active ? ModelService.stop : ModelService.start;
            return action(model.name)
                .catch(err => this.context.notifier.error(`Could not ${action.name} model: ${err.message}`))
                .then(() => this.getModels())
                .finally(() => this.setState(prevState => ({
                deploymentChangingMap: {
                    ...prevState.deploymentChangingMap,
                    [model.name]: false
                }
            })));
        };
        this.onTableChange = (criteria) => {
            if (criteria.sort) {
                this.setState({ sort: criteria.sort });
            }
        };
        this.sortModels = (sort, models) => {
            const sortedModels = sort.field === 'created' ? sortBy(models, model => moment(model.created)) : sortBy(models, sort.field);
            return sort.direction === 'asc' ? sortedModels : sortedModels.reverse();
        };
        props.updateHeader({
            currentTab: 'models',
            headerActions: (<EuiHeaderLink href={'#/model/savedSearchSelection'} data-test="new-model">New</EuiHeaderLink>)
        });
        this.columns = [
            {
                name: 'Name',
                field: 'name',
                sortable: true,
                render: name => <EuiLink href={`#/model/${name}/view`} data-test={name}>{name}</EuiLink>
            }, {
                name: 'Type',
                field: 'type',
                sortable: true
            }, {
                name: 'State',
                field: 'training.state',
                sortable: true,
                render: (state, model) => {
                    if (state === 'failed') {
                        return (<React.Fragment>
                Training failed. See
                <EuiLink href={`#/model/${model.name}/view`} style={{ marginLeft: '4px' }}>here</EuiLink>.
              </React.Fragment>);
                    }
                    else if (trainingInProgressStatuses.includes(state)) {
                        return <EuiProgress value={model.training.progress} max={100} color="primary"/>;
                    }
                    else {
                        return state.charAt(0).toUpperCase() + state.slice(1);
                    }
                }
            }, {
                name: 'Created',
                field: 'created',
                sortable: true,
                render: created => (<EuiToolTip position="right" content={`${moment.utc(created).format('HH:mm:ss')} UTC`}>
            <React.Fragment>
              {moment.utc(created).format('YYYY-MM-DD')}
            </React.Fragment>
          </EuiToolTip>)
            }, {
                field: 'name',
                name: 'Actions',
                truncateText: false,
                render: (name, model) => (<EuiFlexGroup alignItems="center" gutterSize="s">
            <EuiFlexItem grow={false}>
              <EuiToolTip content={`${model.active ? 'Stop' : 'Start'} live ${model.type === 'Anomaly' ? 'detection of anomalies' : 'prediction'}`}>
                <EuiSwitch showLabel={false} label="Start/stop" data-test={`toggle-${model.name}`} checked={model.active} disabled={this.state.deploymentChangingMap[name] || model.training.state !== 'trained'} onChange={() => this.toggleStart(model)}/>
              </EuiToolTip>
            </EuiFlexItem>
            <EuiFlexItem grow={false}>
              <EuiLoadingSpinner size="s" style={{ visibility: this.state.deploymentChangingMap[name] ? 'visible' : 'hidden' }}/>
            </EuiFlexItem>
            <EuiFlexItem grow={false} style={{ visibility: model.type === 'Anomaly' ? 'visible' : 'hidden' }}>
              <EuiToolTip content="Historical Detection">
                <EuiButtonIcon iconType="clock" color="primary" isDisabled={model.training.state !== 'trained'} aria-label="historicalDetection" href={`#/model/${name}/historicalDetection`}/>
              </EuiToolTip>
            </EuiFlexItem>
            <EuiFlexItem grow={false}>
              <EuiToolTip content="Delete the model">
                <EuiButtonIcon iconType="trash" color="danger" aria-label="remove" isDisabled={this.state.deletingInProgressMap[name]} onClick={() => this.deleteModel(name)}/>
              </EuiToolTip>
            </EuiFlexItem>
          </EuiFlexGroup>)
            }
        ];
        this.state = {
            deploymentChangingMap: {},
            deletingInProgressMap: {},
            sort: {
                field: 'name',
                direction: 'asc'
            }
        };
        setMlPageTitle('Models');
    }
    componentDidMount() {
        this.getModels()
            .then(() => {
            this.setIntervalRef = setInterval(() => {
                if (this.state.models && this.state.models.some(model => trainingInProgressStatuses.includes(model.training.state))) {
                    return this.getModels()
                        .catch(() => clearInterval(this.setIntervalRef));
                }
            }, 10000);
        });
    }
    componentWillUnmount() {
        clearInterval(this.setIntervalRef);
    }
    getModels() {
        return ModelService.getAll()
            .then(models => this.setState({ models }))
            .catch(err => {
            this.context.notifier.error(`Could not get models: ${err.message}`);
            this.setState({
                models: [],
                error: err.message
            });
        });
    }
    render() {
        return (<React.Fragment>
        <EuiPageContentHeader>
          <EuiPageContentHeaderSection>
            <EuiTitle>
              <h2>Models</h2>
            </EuiTitle>
          </EuiPageContentHeaderSection>
        </EuiPageContentHeader>

        {this.renderModels()}
      </React.Fragment>);
    }
    renderModels() {
        const sorting = {
            sort: {
                field: this.state.sort.field,
                direction: this.state.sort.direction
            }
        };
        return (<EuiBasicTable items={this.state.models ? this.sortModels(sorting.sort, this.state.models) : []} columns={this.columns} loading={!this.state.models} noItemsMessage={this.state.models ? 'No models found' : 'Loading models...'} onChange={this.onTableChange} sorting={sorting} error={this.state.error} pagination={undefined}/>);
    }
}
ModelList.contextType = AppDependencies;
