import Service from './Service';
import { get, cloneDeep } from 'lodash';
export default class ElasticsearchService extends Service {
    static query(index, filters = [], timeRange, size = 10, sortBy, searchQuery = { query: { bool: { must: [], must_not: [] } } }) {
        const query = {
            query: cloneDeep(searchQuery.query),
            size
        };
        query.query.bool.must.push(...filters.map(filter => ({
            match: {
                [filter.field]: filter.value
            }
        })));
        query.size = size;
        if (sortBy) {
            query.sort = sortBy;
        }
        if (timeRange) {
            query.query.bool.must.push({
                range: {
                    [timeRange.field]: {
                        gte: +timeRange.start,
                        lt: +timeRange.end
                    }
                }
            });
        }
        return this.http.post(`../../elasticsearch/${index}/_search`, query)
            .then(resp => resp.data.hits.hits.map(hit => hit._source))
            .catch(resp => {
            if (resp instanceof Error) {
                throw resp;
            }
            else {
                throw new Error(resp.error.reason);
            }
        });
    }
    static statField(index, field, filters = []) {
        const query = {
            query: {
                bool: {
                    must: [
                        ...filters.map(filter => ({
                            match: {
                                [filter.field]: filter.value
                            }
                        }))
                    ]
                }
            },
            aggs: {
                stat_agg: {
                    stats: { field }
                }
            },
            size: 0
        };
        return this.http.post(`../../elasticsearch/${index}/_search`, query)
            .then(resp => {
            if (resp.data.aggregations) {
                return resp.data.aggregations.stat_agg;
            }
            else {
                throw new Error('No indices found for this index pattern');
            }
        })
            .catch(resp => {
            if (resp instanceof Error) {
                throw resp;
            }
            else {
                throw new Error(resp.data.error.reason);
            }
        });
    }
    static dateHistogram(index, startTime, endTime, aggInterval, timeField, metrics, searchQuery = { query: { bool: { must: [], must_not: [] } } }) {
        const query = {
            query: cloneDeep(searchQuery.query),
            size: 0,
            aggs: {
                timeseries: {
                    date_histogram: {
                        field: timeField,
                        interval: aggInterval,
                        'min_doc_count': 0,
                        format: 'epoch_millis',
                        'extended_bounds': {
                            min: +startTime,
                            max: +endTime
                        }
                    }
                }
            }
        };
        query.query.bool.must.push({
            range: { [timeField]: { gte: +startTime, lte: +endTime, format: 'epoch_millis' } }
        });
        if (metrics) {
            query.aggs.timeseries.aggs = {};
            for (const { field, aggType } of metrics) {
                if (aggType !== 'count') {
                    query.aggs.timeseries.aggs[`aggregation-${field}`] = { [aggType]: { field } };
                }
            }
        }
        return this.http.post(`../../elasticsearch/${index}/_search`, query)
            .catch(err => Promise.reject(new Error(get(err, 'error.reason', err))))
            .then(resp => !resp.data.aggregations || !resp.data.hits.total.value ? [] : resp.data.aggregations.timeseries.buckets.map(bucket => {
            if (metrics) {
                return [
                    bucket.key,
                    ...metrics.map(({ field, aggType }) => aggType === 'count' ? bucket.doc_count : bucket[`aggregation-${field}`].value)
                ];
            }
            else {
                return [bucket.key, bucket.doc_count];
            }
        }));
    }
    static histogram(index, term, interval, aggType, fields, searchQuery = { query: { bool: { must: [], must_not: [] } } }) {
        const query = {
            query: searchQuery.query,
            size: 0,
            aggs: {
                histogramAgg: {
                    histogram: {
                        field: term,
                        interval
                    }
                }
            }
        };
        if (aggType !== 'count') {
            query.aggs.histogramAgg.aggs = {};
            for (const field of fields) {
                query.aggs.histogramAgg.aggs[field] = {
                    [aggType]: {
                        field: field
                    }
                };
            }
        }
        return this.http.post(`../../elasticsearch/${index}/_search`, query)
            .then(resp => !resp.data.aggregations ? [] : resp.data.aggregations.histogramAgg.buckets.map(bucket => [
            bucket.key,
            ...(aggType === 'count' ? [bucket.doc_count] : fields.map(field => bucket[field].value))
        ]))
            .catch(err => Promise.reject(new Error(get(err, 'error.reason', err))));
    }
}
