import React, { PureComponent } from 'react';
import { ElasticsearchService } from '../services';
import { dateMathToMilliseconds, dateTickFormatter, getInterval } from '../TimeUtils';
import { COLOURS, getMlOutputIndex } from '../constants';
import { Graph } from '../generic_components';
import { ScaleType } from '@elastic/charts';
import moment from 'moment';
import { predictionDataBuilder } from './PredictionDataBuilder';
export class ExplorerVisGraph extends PureComponent {
    constructor(props) {
        super(props);
        this.getTimeParameters = () => {
            const actualBucketSize = getInterval(this.props.model.data.interval, this.props.startTime, this.props.endTime);
            const intervalAsMillis = dateMathToMilliseconds(actualBucketSize);
            const appliedStartTimeAsMillis = Math.floor(this.props.startTime.valueOf() / intervalAsMillis) * intervalAsMillis;
            const appliedEndTimeeAsMillis = Math.ceil(this.props.endTime.valueOf() / intervalAsMillis) * intervalAsMillis;
            return {
                actualBucketSize,
                appliedStartTime: moment(appliedStartTimeAsMillis),
                appliedEndTime: moment(appliedEndTimeeAsMillis === appliedStartTimeAsMillis ? appliedEndTimeeAsMillis : appliedEndTimeeAsMillis - 1)
            };
        };
        this.getSourceData = () => {
            const timeParameters = this.getTimeParameters();
            const query = {
                query: {
                    bool: {
                        must: this.props.model.data.filters.must,
                        must_not: this.props.model.data.filters.mustNot
                    }
                }
            };
            return ElasticsearchService.dateHistogram(this.props.model.data.index, timeParameters.appliedStartTime, timeParameters.appliedEndTime, timeParameters.actualBucketSize, this.props.model.data.timeField, [{ field: this.props.model.inputs[0], aggType: this.props.model.data.aggregationType }], query);
        };
        this.getData = () => {
            this.setState({ errorMessage: undefined });
            if (this.props.model.type === 'Anomaly') {
                return this.getDataForAnomalyModel();
            }
            else {
                return this.getDataForPredictionModel();
            }
        };
        this.state = {
            isLoading: true
        };
    }
    componentDidMount() {
        this.getData();
    }
    componentDidUpdate(prevProps) {
        if (prevProps.model !== this.props.model || prevProps.startTime !== this.props.startTime || prevProps.endTime !== this.props.endTime) {
            this.getData();
        }
    }
    render() {
        const seriesLabels = this.props.model ? [this.props.model.inputs[0]] : [];
        if (this.props.model && this.props.model.type === 'Prediction') {
            seriesLabels.push(`${this.props.model.inputs[0]}-prediction`);
        }
        return (<Graph isLoading={this.state.isLoading} errorMessage={this.state.errorMessage} style={{ height: 'calc(100% - 40px)' }} xLabel={this.props.model ? this.props.model.data.timeField : ''} yLabel={this.props.model ? this.props.model.inputs[0] : ''} xLabelType={ScaleType.Time} seriesLabels={this.props.model.data.aggregationType === 'count' ? ['count'] : seriesLabels} lineColours={[COLOURS.blue, COLOURS.yellow]} areaColour={this.props.model.type === 'Prediction' ? COLOURS.lightyellow : undefined} data={this.state.data} area={this.state.area} areaLabel={this.props.model.type === 'Anomaly' ? 'Expected range' : 'Predicted range'} annotations={this.state.annotations} onBrushEnd={this.props.onBrushEnd}/>);
    }
    getDataForAnomalyModel() {
        return Promise.all([
            this.getSourceData(),
            this.getAnomalyDetectionData()
        ])
            .then(([data, { area, annotations }]) => this.setState({ isLoading: false, data, area, annotations }))
            .catch(err => this.setState({ isLoading: false, errorMessage: err.message }));
    }
    getAnomalyDetectionData() {
        const timeParameters = this.getTimeParameters();
        return ElasticsearchService.dateHistogram(getMlOutputIndex(this.props.model.name), timeParameters.appliedStartTime, timeParameters.appliedEndTime, timeParameters.actualBucketSize, `ref_timestamp`, [
            { field: `${this.props.model.inputs[0]}.${this.props.model.data.aggregationType}.lower_bound`, aggType: 'avg' },
            { field: `${this.props.model.inputs[0]}.${this.props.model.data.aggregationType}.upper_bound`, aggType: 'avg' },
            { field: `${this.props.model.inputs[0]}.${this.props.model.data.aggregationType}.anomaly_score`, aggType: 'max' }
        ])
            .then(data => {
            const area = data.map(prediction => prediction.slice(0, 3));
            const annotations = data
                .filter(prediction => prediction[3] > 1.5)
                .map(prediction => ({
                severity: (prediction[3] > 2 ? 'critical' : 'warning'),
                dataValue: prediction[0],
                details: String(prediction[1]),
                header: dateTickFormatter(prediction[0], timeParameters.appliedStartTime, timeParameters.appliedEndTime)
            }));
            return { area, annotations };
        });
    }
    getDataForPredictionModel() {
        return Promise.all([
            this.getSourceData(),
            this.getPredictionData()
        ])
            .then(([data, predictionData]) => {
            const formattedData = predictionDataBuilder(data, predictionData);
            this.setState({
                isLoading: false,
                data: formattedData.line,
                area: formattedData.area,
                annotations: undefined
            });
        })
            .catch(err => this.setState({ isLoading: false, errorMessage: err.message }));
    }
    getPredictionData() {
        const timeParameters = this.getTimeParameters();
        return ElasticsearchService.dateHistogram(getMlOutputIndex(this.props.model.name), timeParameters.appliedStartTime, timeParameters.appliedEndTime, timeParameters.actualBucketSize, `ref_timestamp`, [
            { field: `${this.props.model.inputs[0]}.${this.props.model.data.aggregationType}.lower_bound`, aggType: 'avg' },
            { field: `${this.props.model.inputs[0]}.${this.props.model.data.aggregationType}.upper_bound`, aggType: 'avg' },
            { field: `${this.props.model.inputs[0]}.${this.props.model.data.aggregationType}.prediction`, aggType: 'avg' }
        ])
            .then(predictions => {
            const predictionArea = predictions.map(prediction => prediction.slice(0, 3));
            const predictionLine = predictions.map(prediction => [prediction[0], prediction[3]]);
            return { predictionArea, predictionLine };
        });
    }
}
