"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const moment = require('moment'); // eslint-disable-line @typescript-eslint/no-var-requires
const Routes_1 = require("./Routes");
const lodash_1 = require("lodash");
class ModelRoutes extends Routes_1.default {
    registerRoutes(server) {
        server.route({
            method: 'GET',
            path: '/',
            handler: (req, reply) => this.sirenMl.models.getAll()
                .then(models => Promise.all(models.map(this.convertToApiModelSummary.bind(this))))
                .then(reply)
                .catch(reply)
        });
        server.route({
            method: 'GET',
            path: '/{id}',
            handler: (req, reply) => this.sirenMl.models.get(req.params.id)
                .then(model => this.convertToApiModel(model))
                .then(reply)
                .catch(reply)
        });
        server.route({
            method: 'PUT',
            path: '/',
            handler: (req, reply) => this.create(req.payload, lodash_1.get(req, 'auth.credentials.credentials'))
                .then(() => reply().code(201))
                .catch(reply)
        });
        server.route({
            method: 'POST',
            path: '/{id}',
            handler: (req, reply) => this.update(req.payload, req.query.isUpdate === 'true', lodash_1.get(req, 'auth.credentials.credentials'))
                .then(reply)
                .catch(reply)
        });
        server.route({
            method: 'POST',
            path: '/{id}/_start',
            handler: (req, reply) => this.sirenMl.deployment.activateLive(req.params.id).then(reply).catch(reply)
        });
        server.route({
            method: 'POST',
            path: '/{id}/_stop',
            handler: (req, reply) => this.sirenMl.deployment.stop(req.params.id).then(reply).catch(reply)
        });
        server.route({
            method: 'POST',
            path: '/{id}/predict',
            handler: (req, reply) => this.sirenMl.deployment.activateHistorical(req.params.id, req.payload).then(reply).catch(reply)
        });
        server.route({
            method: 'DELETE',
            path: '/{id}',
            handler: (req, reply) => this.sirenMl.models.delete(req.params.id).then(reply).catch(reply)
        });
    }
    async convertToApiModel(model) {
        const splittedInput = model.training.config.data.seq_inputs[0].split('.');
        return {
            name: model.name,
            id: model.model_id,
            type: model.type,
            training: {
                state: model.training.status,
                progress: model.training.status === 'trained' ? 100 : await this.getProgressOfTask(model.training.task_id),
                error: model.training.error
            },
            active: !!model.deployment.task_id,
            data: {
                index: model.training.config.data.source.index,
                timeField: model.training.config.data.source.aggregation.field,
                aggregationType: model.training.config.data.seq_inputs.includes('count') ?
                    'count' : splittedInput[splittedInput.length - 1],
                interval: model.training.config.data.source.aggregation.interval,
                savedSearch: model.training.config.data.source.savedSearch,
                start: model.training.config.data.source.ranges.gte,
                end: model.training.config.data.source.ranges.lte,
                filters: model.training.config.data.source.filters
            },
            inputs: model.training.config.data.seq_inputs
                .filter(input => input !== 'count')
                .map(input => input.split('.').slice(0, -1).join('.')),
            inputSize: model.training.config.data.input_seq_len,
            outputSize: model.training.config.data.output_seq_len,
            created: model.created
        };
    }
    async convertToApiModelSummary(model) {
        return {
            name: model.name,
            type: model.type,
            training: {
                state: model.training.status,
                progress: model.training.status === 'trained' ? 100 : await this.getProgressOfTask(model.training.task_id),
            },
            active: !!model.deployment.task_id,
            created: model.created
        };
    }
    getProgressOfTask(taskId) {
        return this.sirenMl.jobs.get(taskId)
            .then(job => job.progress)
            .catch(() => undefined);
    }
    static convertToSirenMlModelCreationConfig(model, isUpdate) {
        const inputs = model.data.aggregationType === 'count' ? ['count'] : model.inputs.map(input => `${input}.${model.data.aggregationType}`);
        const steps = isUpdate ? ['update', 'test'] : ['hpopt', 'train', 'test'];
        return {
            name: model.name,
            steps: steps,
            type: model.type,
            data: {
                source: {
                    name: model.name,
                    source: 'es',
                    index: model.data.index,
                    savedSearch: model.data.savedSearch,
                    method: 'conditional',
                    aggregation: {
                        field: model.data.timeField,
                        interval: model.data.interval,
                        type: 'date_histogram'
                    },
                    filters: model.data.filters,
                    ranges: {
                        field: model.data.timeField,
                        gte: moment(model.data.start).toISOString(),
                        lte: moment(model.data.end).toISOString()
                    }
                },
                inputs: inputs.concat('timestamp'),
                seq_inputs: inputs,
                input_seq_len: model.inputSize,
                outputs: inputs,
                seq_outputs: inputs,
                output_seq_len: model.outputSize || 1,
                seq_offset: 1
            }
            // settings: {
            //   train: {
            //     iterations: 100,
            //     batch_size: 64
            //   },
            //   hpopt: {
            //     max_trials: 5,
            //     min_trials_to_use: 3
            //   }
            // }
        };
    }
    async create(model, credentials) {
        return this.sirenMl.models.create(ModelRoutes.convertToSirenMlModelCreationConfig(model, false), credentials);
    }
    update(model, isUpdate, credentials) {
        return this.sirenMl.models.update(ModelRoutes.convertToSirenMlModelCreationConfig(model, isUpdate), credentials);
    }
}
exports.default = ModelRoutes;
